﻿using System;
using System.IO;
using System.Windows.Forms;

using TestClient.FileService;

namespace TestClient
{
    /// <summary>
    /// 메인 폼
    /// </summary>
    public partial class MainForm : Form
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - MainForm()

        /// <summary>
        /// 생성자
        /// </summary>
        public MainForm()
        {
            InitializeComponent();

            this.uploadButton.Click   += uploadButton_Click;
            this.downloadButton.Click += downloadButton_Click;
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Private
        //////////////////////////////////////////////////////////////////////////////// Event

        #region 업로드 버튼 클릭시 처리하기 - uploadButton_Click(sender, e)

        /// <summary>
        /// 업로드 버튼 클릭시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void uploadButton_Click(object sender, EventArgs e)
        {
            using(FileServiceClient client = new FileServiceClient("NetTcpBinding_IFileService"))
            {
                FileInfo sourceFileInfo = new FileInfo("c:\\sample.txt");

                using(FileStream sourceStream = File.OpenRead(sourceFileInfo.FullName))
                {
                    using(ReportStream reportStream = new ReportStream(sourceStream, sourceFileInfo.Length, ProgressChanged))
                    {
                        client.Upload(sourceFileInfo.Length, sourceFileInfo.FullName, reportStream);
                    }
                }

                this.progressLabel.Text = string.Empty;
            }
        }

        #endregion
        #region 다운로드 버튼 클릭시 처리하기 - downloadButton_Click(sender, e)

        /// <summary>
        /// 다운로드 버튼 클릭시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void downloadButton_Click(object sender, EventArgs e)
        {
            string targetDirectoryPath = "c:\\target";

            if(!Directory.Exists(targetDirectoryPath))
            {
                Directory.CreateDirectory(targetDirectoryPath);
            }

            using(FileServiceClient client = new FileServiceClient("NetTcpBinding_IFileService"))
            {
                string sourceFileName = "sample.txt";
                string filePath;
                Stream sourceStream;

                long fileLength = client.Download(sourceFileName, out filePath, out sourceStream);

                string targetFilePath = Path.Combine(targetDirectoryPath, sourceFileName);

                using(FileStream targetStream = File.Create(targetFilePath))
                {
                    using(ReportStream reportStream = new ReportStream(sourceStream, fileLength, ProgressChanged))
                    {
                        reportStream.CopyTo(targetStream);
                    }
                }

                this.progressLabel.Text = string.Empty;
            }
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////// Function

        #region 진행 변경시 처리하기 - ProgressChanged(totalCount, totalReadCount)

        /// <summary>
        /// 진행 변경시 처리하기
        /// </summary>
        /// <param name="totalCount">전체 카운트</param>
        /// <param name="totalReadCount">전체 읽은 카운트</param>
        private void ProgressChanged(long totalCount, long totalReadCount)
        {
            long value = 100L * totalReadCount / totalCount;

            this.progressLabel.Text = string.Format("{0}%", value);

            this.progressLabel.Update();
        }

        #endregion
    }
}