﻿using System;
using System.IO;

namespace TestClient
{
    /// <summary>
    /// 리포트 스트림
    /// </summary>
    public class ReportStream : Stream
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// 기준 스트림
        /// </summary>
        private readonly Stream baseStream;

        /// <summary>
        /// 전체 크기
        /// </summary>
        private long totalCount;

        /// <summary>
        /// 리포트 콜백
        /// </summary>
        private readonly Action<long, long> reportCallback;

        /// <summary>
        /// 전체 읽기 카운트
        /// </summary>
        private long totalReadCount;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Property
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 읽기 가능 여부 - CanRead

        /// <summary>
        /// 읽기 가능 여부
        /// </summary>
        public override bool CanRead
        {
            get
            {
                return this.baseStream.CanRead;
            }
        }

        #endregion
        #region 위치 설정 가능 여부 - CanSeek

        /// <summary>
        /// 위치 설정 가능 여부
        /// </summary>
        public override bool CanSeek
        {
            get
            {
                throw new NotImplementedException();
            }
        }

        #endregion
        #region 쓰기 가능 여부 - CanWrite

        /// <summary>
        /// 쓰기 가능 여부
        /// </summary>
        public override bool CanWrite
        {
            get
            {
                throw new NotImplementedException();
            }
        }

        #endregion
        #region 길이 - Length

        /// <summary>
        /// 길이
        /// </summary>
        public override long Length
        {
            get
            {
                return this.baseStream.Length;
            }
        }

        #endregion
        #region 위치 - Position

        /// <summary>
        /// 위치
        /// </summary>
        public override long Position
        {
            get
            {
                throw new NotImplementedException();
            }
            set
            {
                throw new NotImplementedException();
            }
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - ReportStream(baseStream, totalCount, reportCallback)

        /// <summary>
        /// 생성자
        /// </summary>
        /// <param name="baseStream">기준 스트림</param>
        /// <param name="totalCount">전체 카운트</param>
        /// <param name="reportCallback">리포트 콜백</param>
        public ReportStream(Stream baseStream, long totalCount, Action<long, long> reportCallback)
        {
            this.baseStream     = baseStream ?? throw new ArgumentNullException("baseStream");
            this.totalCount     = totalCount;
            this.reportCallback = reportCallback;
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 닫기 - Close()

        /// <summary>
        /// 닫기
        /// </summary>
        public override void Close()
        {
            base.Close();

            this.baseStream.Close();
        }

        #endregion
        #region 버퍼 비우기- Flush()

        /// <summary>
        /// 버퍼 비우기
        /// </summary>
        public override void Flush()
        {
            throw new NotImplementedException();
        }

        #endregion
        #region 위치 설정하기 - Seek(offset, seekOrigin)

        /// <summary>
        /// 위치 설정하기
        /// </summary>
        /// <param name="offset">오프셋</param>
        /// <param name="seekOrigin">위치 기준</param>
        /// <returns>위치</returns>
        public override long Seek(long offset, SeekOrigin seekOrigin)
        {
            throw new NotImplementedException();
        }

        #endregion
        #region 길이 설정하기 - SetLength(value)

        /// <summary>
        /// 길이 설정하기
        /// </summary>
        /// <param name="value">값</param>
        public override void SetLength(long value)
        {
            throw new NotImplementedException();
        }

        #endregion
        #region 읽기 - Read(bufferArray, offset, count)

        /// <summary>
        /// 읽기
        /// </summary>
        /// <param name="bufferArray">버퍼 배열</param>
        /// <param name="offset">오프셋</param>
        /// <param name="count">카운트</param>
        /// <returns>실제 읽은 카운트</returns>
        public override int Read(byte[] bufferArray, int offset, int count)
        {
            int readCount = this.baseStream.Read(bufferArray, offset, count);

            this.totalReadCount += readCount;

            this.reportCallback?.Invoke(this.totalCount, this.totalReadCount);

            return readCount;
        }

        #endregion
        #region 쓰기 - Write(bufferArray, offset, count)

        /// <summary>
        /// 쓰기
        /// </summary>
        /// <param name="bufferArray">버퍼 배열</param>
        /// <param name="offset">오프셋</param>
        /// <param name="count">카운트</param>
        public override void Write(byte[] bufferArray, int offset, int count)
        {
            throw new NotImplementedException();
        }

        #endregion
    }
}