﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.ServiceModel;

namespace TestServer
{
    /// <summary>
    /// 파일 서비스
    /// </summary>
    public class FileService : IFileService
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 모든 파일명 리스트 구하기 - GetAllFileNameList()

        /// <summary>
        /// 모든 파일명 리스트 구하기
        /// </summary>
        /// <returns>모든 파일명 리스트</returns>
        public List<string> GetAllFileNameList()
        {
            List<string> fileNameList = Directory.GetFiles("d:\\upload").Select(Path.GetFileName).ToList();

            return fileNameList;
        }

        #endregion
        #region 업로드 하기 - Upload(fileData)

        /// <summary>
        /// 업로드 하기
        /// </summary>
        /// <param name="fileData">파일 데이터</param>
        public void Upload(FileData fileData)
        {
            string sourceFilePath      = fileData.FilePath;
            string sourceFileName      = Path.GetFileName(sourceFilePath);
            string targetDirectoryPath = "d:\\upload";
            string targetFilePath      = Path.Combine(targetDirectoryPath, sourceFileName);

            if(!Directory.Exists(targetDirectoryPath))
            {
                Directory.CreateDirectory(targetDirectoryPath);
            }

            try
            {
                using(FileStream targetStream = File.Create(targetFilePath))
                {
                    using(Stream sourceStream = fileData.SourceStream)
                    {
                        sourceStream.CopyTo(targetStream);
                    }
                }
            }
            catch(IOException exception)
            {
                Console.WriteLine(exception.Message);

                if(File.Exists(targetFilePath))
                {
                    try
                    {
                        File.Delete(targetFilePath);
                    }
                    catch
                    {
                    }
                }
            }
        }

        #endregion
        #region 다운로드 하기 - Download(fileRequest)

        /// <summary>
        /// 다운로드 하기
        /// </summary>
        /// <param name="fileRequest">파일 요청</param>
        /// <returns>파일 데이터</returns>
        public FileData Download(FileRequest fileRequest)
        {
            string sourceFilePath = Path.Combine("d:\\upload", fileRequest.FileName);

            Stream sourceStream = null;

            try
            {
                sourceStream = File.Open(sourceFilePath, FileMode.Open, FileAccess.Read, FileShare.Read);
            }
            catch(Exception)
            {
                return null;
            }

            OperationContext clientContext = OperationContext.Current;

            clientContext.OperationCompleted += delegate(object sender, EventArgs e)
            {
                if(sourceStream != null)
                {
                    sourceStream.Dispose();
                }
            };

            return new FileData
            {
                FilePath     = fileRequest.FileName,
                FileLength   = sourceStream.Length,
                SourceStream = sourceStream
            };
        }

        #endregion
    }
}